@@ -321,7 +321,7 @@ class MultiLabelTextProcessor(TextProcessor):
             return list(
                 df.apply(
                     lambda row: InputExample(
-                        guid=row.index, text_a=row[text_col], label=[]
+                        guid=row.index, text_a=str(row[text_col]), label=[]
                     ),
                     axis=1,
                 )
@@ -331,7 +331,7 @@ class MultiLabelTextProcessor(TextProcessor):
                 df.apply(
                     lambda row: InputExample(
                         guid=row.index,
-                        text_a=row[text_col],
+                        text_a=str(row[text_col]),
                         label=_get_labels(row, label_col),
                     ),
                     axis=1,
@@ -376,6 +376,8 @@ class BertDataBunch(object):
         clear_cache=False,
         no_cache=False,
         custom_sampler=None,
+        pos_weight=None,
+        weight=None,
     ):
 
         # just in case someone passes string instead of Path
@@ -406,6 +408,8 @@ class BertDataBunch(object):
         self.model_type = model_type
         self.output_mode = "classification"
         self.custom_sampler = custom_sampler
+        self.pos_weight = pos_weight
+        self.weight = weight
         if logger is None:
             logger = logging.getLogger()
         self.logger = logger
@@ -428,7 +432,7 @@ class BertDataBunch(object):
             cached_features_file = os.path.join(
                 self.cache_dir,
                 "cached_{}_{}_{}_{}_{}".format(
-                    self.model_type,
+                    self.model_type.replace("/", "-"),
                     "train",
                     "multi_label" if self.multi_label else "multi_class",
                     str(self.max_seq_length),
@@ -462,7 +466,7 @@ class BertDataBunch(object):
             cached_features_file = os.path.join(
                 self.cache_dir,
                 "cached_{}_{}_{}_{}_{}".format(
-                    self.model_type,
+                    self.model_type.replace("/", "-"),
                     "dev",
                     "multi_label" if self.multi_label else "multi_class",
                     str(self.max_seq_length),
@@ -518,10 +522,10 @@ class BertDataBunch(object):
             test_examples, "test", is_test=True, no_cache=True
         )
 
+        batch_size = self.batch_size_per_gpu * 2 * max(1, self.n_gpu)
+
         test_sampler = SequentialSampler(test_dataset)
-        return DataLoader(
-            test_dataset, sampler=test_sampler, batch_size=self.batch_size_per_gpu
-        )
+        return DataLoader(test_dataset, sampler=test_sampler, batch_size=batch_size)
 
     def save(self, filename="databunch.pkl"):
         tmp_path = self.data_dir / "tmp"
@@ -545,7 +549,7 @@ class BertDataBunch(object):
         cached_features_file = os.path.join(
             self.cache_dir,
             "cached_{}_{}_{}_{}_{}".format(
-                self.model_type,
+                self.model_type.replace("/", "-"),
                 set_type,
                 "multi_label" if self.multi_label else "multi_class",
                 str(self.max_seq_length),
