@@ -10,6 +10,15 @@ from transformers import (
     DistilBertForSequenceClassification,
     CamembertForSequenceClassification,
     AlbertForSequenceClassification,
+    ElectraForSequenceClassification,
+    XLMModel,
+    XLMConfig,
+    XLMTokenizer, 
+    XLMForSequenceClassification,
+    FlaubertModel,
+    FlaubertConfig,
+    FlaubertTokenizer,
+    FlaubertForSequenceClassification,
 )
 
 import torch
@@ -47,7 +56,13 @@ class DistilBertForMultiLabelSequenceClassification(
         loss, logits = outputs[:2]
     """
 
-    def forward(self, input_ids, attention_mask=None, labels=None, head_mask=None):
+    def forward(
+            self,
+            input_ids,
+            attention_mask=None,
+            labels=None,
+            head_mask=None
+    ):
         distilbert_output = self.distilbert(
             input_ids=input_ids, attention_mask=attention_mask, head_mask=head_mask
         )
@@ -61,7 +76,7 @@ class DistilBertForMultiLabelSequenceClassification(
 
         outputs = (logits,) + distilbert_output[1:]
         if labels is not None:
-            loss_fct = BCEWithLogitsLoss()
+            loss_fct = BCEWithLogitsLoss(weight=self.weight, pos_weight=self.pos_weight)
 
             loss = loss_fct(
                 logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
@@ -121,7 +136,7 @@ class RobertaForMultiLabelSequenceClassification(RobertaForSequenceClassificatio
         outputs = (logits,) + outputs[2:]
 
         if labels is not None:
-            loss_fct = BCEWithLogitsLoss()
+            loss_fct = BCEWithLogitsLoss(weight=self.weight, pos_weight=self.pos_weight)
 
             loss = loss_fct(
                 logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
@@ -199,7 +214,7 @@ class BertForMultiLabelSequenceClassification(BertForSequenceClassification):
         outputs = (logits,) + outputs[2:]
 
         if labels is not None:
-            loss_fct = BCEWithLogitsLoss()
+            loss_fct = BCEWithLogitsLoss(weight=self.weight, pos_weight=self.pos_weight)
 
             loss = loss_fct(
                 logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
@@ -274,7 +289,7 @@ class XLNetForMultiLabelSequenceClassification(XLNetForSequenceClassification):
         outputs = (logits,) + transformer_outputs[1:]
 
         if labels is not None:
-            loss_fct = BCEWithLogitsLoss()
+            loss_fct = BCEWithLogitsLoss(weight=self.weight, pos_weight=self.pos_weight)
 
             loss = loss_fct(
                 logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
@@ -307,7 +322,7 @@ class AlbertForMultiLabelSequenceClassification(AlbertForSequenceClassification)
         outputs = (logits,) + outputs[2:]
 
         if labels is not None:
-            loss_fct = BCEWithLogitsLoss()
+            loss_fct = BCEWithLogitsLoss(weight=self.weight, pos_weight=self.pos_weight)
 
             loss = loss_fct(
                 logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
@@ -339,6 +354,41 @@ class CamembertForMultiLabelSequenceClassification(CamembertForSequenceClassific
 
         outputs = (logits,) + outputs[2:]
 
+        if labels is not None:
+            loss_fct = BCEWithLogitsLoss(weight=self.weight, pos_weight=self.pos_weight)
+
+            loss = loss_fct(
+                logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
+            )
+            outputs = (loss,) + outputs
+
+        return outputs  # (loss), logits, (hidden_states), (attentions)
+
+class ElectraForMultiLabelSequenceClassification(ElectraForSequenceClassification):
+    def forward(
+        self,
+        input_ids,
+        token_type_ids=None,
+        attention_mask=None,
+        labels=None,
+        position_ids=None,
+        head_mask=None,
+        inputs_embeds=None,
+    ):
+        outputs = self.electra(
+            input_ids=input_ids,
+            attention_mask=attention_mask,
+            token_type_ids=token_type_ids,
+            position_ids=position_ids,
+            head_mask=head_mask,
+            inputs_embeds=inputs_embeds
+        )
+
+        sequence_output = outputs[0]
+        logits = self.classifier(sequence_output)
+
+        outputs = (logits,) + outputs[2:]  # add hidden states and attention if they are here
+
         if labels is not None:
             loss_fct = BCEWithLogitsLoss()
 
@@ -348,3 +398,52 @@ class CamembertForMultiLabelSequenceClassification(CamembertForSequenceClassific
             outputs = (loss,) + outputs
 
         return outputs  # (loss), logits, (hidden_states), (attentions)
+
+    
+class FlaubertForMultiLabelSequenceClassification(FlaubertForSequenceClassification):
+    def forward(
+        self,
+        input_ids=None,
+        attention_mask=None,
+        langs=None,
+        token_type_ids=None,
+        position_ids=None,
+        lengths=None,
+        cache=None,
+        head_mask=None,
+        inputs_embeds=None,
+        labels=None,
+        output_attentions=None,
+        output_hidden_states=None,
+    ):
+        transformer_outputs = self.transformer(
+            input_ids,
+            attention_mask=attention_mask,
+            langs=langs,
+            token_type_ids=token_type_ids,
+            position_ids=position_ids,
+            lengths=lengths,
+            cache=cache,
+            head_mask=head_mask,
+            inputs_embeds=inputs_embeds,
+            output_attentions=output_attentions,
+            output_hidden_states=output_hidden_states,
+        )
+
+        output = transformer_outputs[0]
+        logits = self.sequence_summary(output)
+
+        outputs = (logits,) + transformer_outputs[1:]  # Keep new_mems and attention/hidden states if they are here
+
+        # Keep mems, hidden states, attentions if there are in it
+        outputs = (logits,) + transformer_outputs[1:]
+
+        if labels is not None:
+            loss_fct = BCEWithLogitsLoss(weight=self.weight, pos_weight=self.pos_weight)
+
+            loss = loss_fct(
+                logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
+            )
+            outputs = (loss,) + outputs
+
+        return outputs  # (loss), logits, (hidden_states), (attentions)
